<?php

use Chekonline\Cashbox\Api;
use Chekonline\Cashbox\Commands\BatchCommand;
use Chekonline\Cashbox\Commands\ComplexCommand;
use Chekonline\Cashbox\Commands\NoOperationCommand;
use Chekonline\Cashbox\DocumentType;
use Chekonline\Cashbox\Exceptions\ChekonlineCommandException;
use Chekonline\Cashbox\Exceptions\ChekonlineLineException;
use Chekonline\Cashbox\Exceptions\ChekonlineSDKException;
use Chekonline\Cashbox\Line;
use Chekonline\Cashbox\PayAttribute;

if (!defined('_PS_VERSION_'))
{
    exit;
}

require_once dirname(__FILE__).'/classes/ReceiptModel.php';
require_once dirname(__FILE__).'/vendor/autoload.php';
require_once dirname(__FILE__) . '/ChekonlineHelper.php';

/**
 * Main class module
 * Class Chekonline
 */
class Chekonline extends Module
{
    /**
     * Количество дней переотправки чека cron задачей
     */
    const NUMBER_OF_DAYS_FOR_RESEND_RECEIPT = 360;

    /**
     * @var string
     */
    private $logPrefix;

    /**
     * Chekonline constructor.
     */
    public function __construct ()
    {
        $this->name = 'chekonline';
        $this->tab = 'administration';
        $this->version = '1.1.0';
        $this->author = 'Chekonline';
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = 'Чек-онлайн';
        $this->description = 'Чек-онлайн';
        $this->confirmUninstall = 'Are you sure you want to uninstall?';

        if (!Configuration::get('ENDPOINT') ||
            !Configuration::get('PRODUCT_VAT') ||
            !Configuration::get('DELIVERY_VAT')
        ) {
            $this->warning = 'Требуется настройка';
        }

        $this->logPrefix = Configuration::get('LOG_PREFIX');
    }


    /**
     * @return bool
     */
    public function install ()
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        if (!parent::install()) {
            return false;
        }

        if (!Configuration::updateValue('ENDPOINT', '') ||
            !Configuration::updateValue('GROUP', '') ||
            !Configuration::updateValue('CLIENT_ID', '') ||
            !Configuration::updateValue('CASHIER_PASSWORD', '1') ||
            !Configuration::updateValue('DOCS_IN_TURN', '1000') ||
            !Configuration::updateValue('CERT_PASSWORD', '') ||
            !Configuration::updateValue('CERT', '') ||
            !Configuration::updateValue('KEY', '') ||
            !Configuration::updateValue('TAX_MODE', '') ||
            !Configuration::updateValue('PRODUCT_VAT', '') ||
            !Configuration::updateValue('DELIVERY_VAT', '') ||
            !Configuration::updateValue('LOG', '0') ||
            !Configuration::updateValue('LOG_PREFIX', md5(rand(10000, 999999999))
            )
        ) {
            return false;
        }

        if (!$this->registerHook('actionOrderStatusUpdate')) {
            return false;
        }

        if (!$this->installTab(
            'AdminParentOrders',
            'Receipt',
            'Чек-онлайн')
        ) {
            return false;
        }

        // Set conf for all order status
        $sql = 'SELECT * FROM `'._DB_PREFIX_.'order_state_lang`';
        if ($results = Db::getInstance()->ExecuteS($sql)) {
            foreach ($results as $row) {
                $st_var = 'STATUS_PAID_' . $row['id_order_state'];
                if (!Configuration::updateValue($st_var, 0)) {
                    return false;
                }

                $st_var = 'STATUS_RETURN_' . $row['id_order_state'];
                if (!Configuration::updateValue($st_var, 0)) {
                    return false;
                }
            }
        }

        $installSQL = array();
        include dirname(__FILE__).'/install/install.php';
        foreach ($installSQL as $SQL) {
            if (!Db::getInstance()->Execute($SQL)) {
                return false;
            }
        }

        return true;
    }

    /**
     * @return bool
     */
    public function uninstall ()
    {
        // Call uninstall parent method
        if (!parent::uninstall()) {
            return false;
        }

        // Delete conf
        if (!Configuration::deleteByName('ENDPOINT', '') ||
            !Configuration::deleteByName('GROUP', '') ||
            !Configuration::deleteByName('CLIENT_ID', '') ||
            !Configuration::deleteByName('CASHIER_PASSWORD', '') ||
            !Configuration::deleteByName('DOCS_IN_TURN', '') ||
            !Configuration::deleteByName('CERT_PASSWORD', '') ||
            !Configuration::deleteByName('CERT', '') ||
            !Configuration::deleteByName('KEY', '') ||
            !Configuration::deleteByName('TAX_MODE', '') ||
            !Configuration::deleteByName('PRODUCT_VAT', '') ||
            !Configuration::deleteByName('DELIVERY_VAT', '')
        ) {
            return false;
        }

        // Uninstall admin tab
        $this->uninstallTab('Receipt');

        $uninstallSQL = array();
        include dirname(__FILE__).'/install/uninstall.php';
        foreach ($uninstallSQL as $SQL) {
            if (!Db::getInstance()->Execute($SQL)) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param null|string $parent
     * @param string $className
     * @param string $name
     * @return bool
     */
    public function installTab ($parent = null, $className,  $name)
    {
        // Create new admin tab
        $sql = "SELECT id_tab FROM `"._DB_PREFIX_."tab` where class_name ='".$parent."'";
        $result = Db::getInstance()->getValue($sql);


        if ($result == false) {
            return false;
        }

        $tab = new Tab();
        $tab->id_parent = (int)$result;

        $tab->name = array();
        foreach (Language::getLanguages(true) as $lang)
            $tab->name[$lang['id_lang']] = $name;
        $tab->class_name = $className;
        $tab->module = $this->name;
        $tab->active = 1;
        return $tab->add();
    }

    /**
     * @param $className
     * @return bool
     */
    public static function uninstallTab ($className)
    {
        $idTab = (int)Tab::getIdFromClassName($className);
        $tab = new Tab((int)$idTab);
        return $tab->delete();
    }

    /**
     * Settings
     *
     * @return string
     */
    public function getContent ()
    {
        $output = null;

        if (Tools::isSubmit('submit'.$this->name))
        {
            /*
             * Validate and save settings
             */

            $errors = array();


            $key = 'ENDPOINT';
            $value = strval(Tools::getValue($key));
            if (!$value
                || empty($value)) {
                $errors[$key] = 'Некорректное значение свойства адрес устройства/сервера';
            } else {
                Configuration::updateValue($key, $value);
            }


            $key = 'GROUP';
            $value = strval(Tools::getValue($key));

            Configuration::updateValue($key, $value);


            $key = 'CLIENT_ID';
            $value = strval(Tools::getValue($key));
            Configuration::updateValue($key, $value);


            $key = 'CASHIER_PASSWORD';
            $value = strval(Tools::getValue($key));
            if (!$value
                || empty($value)) {
                $errors[$key] = 'Некорректное значение свойства пароль кассира';
            } else {
                Configuration::updateValue($key, $value);
            }

            $key = 'DOCS_IN_TURN';
            $value = strval(Tools::getValue($key));
            if (!$value
                || empty($value)) {
                $errors[$key] = 'Некорректное значение свойства количество документов в смене до автоматического закрытия';
            } else {
                Configuration::updateValue($key, $value);
            }

            $key = 'CERT_PASSWORD';
            $value = strval(Tools::getValue($key));
            Configuration::updateValue($key, $value);


            $key = 'CERT';
            $value = strval(Tools::getValue($key));
            Configuration::updateValue($key, $value);


            $key = 'KEY';
            $value = strval(Tools::getValue($key));
            Configuration::updateValue($key, $value);


            $key = 'TAX_MODE';
            $value = strval(Tools::getValue($key));
            if (isset($value) == false) {
                $errors[$key] = 'Некорректное значение свойства система налогообложения';
            } else {
                Configuration::updateValue($key, $value);
            }

            $key = 'PRODUCT_VAT';
            $value = strval(Tools::getValue($key));
            if (!$value
                || empty($value)) {
                $errors[$key] = 'Некорректное значение свойства НДС на товары';
            } else {
                Configuration::updateValue($key, $value);
            }

            $key = 'DELIVERY_VAT';
            $value = strval(Tools::getValue($key));
            if (!$value
                || empty($value)) {
                $errors[$key] = 'Некорректное значение свойства НДС на доставку';
            } else {
                Configuration::updateValue($key, $value);
            }


            $sql = 'SELECT * FROM `'._DB_PREFIX_.'order_state_lang`';
            if ($results = Db::getInstance()->ExecuteS($sql)) {
                foreach ($results as $row) {
                    $st_var = 'STATUS_PAID_' . $row['id_order_state'];
                    Configuration::updateValue($st_var, Tools::getValue($st_var));

                    $st_var = 'STATUS_RETURN_' . $row['id_order_state'];
                    Configuration::updateValue($st_var, Tools::getValue($st_var));
                }
            }

            $key = 'LOG';
            $formKey = 'LOG_LOG';
            $value = strval(Tools::getValue($formKey));
            Configuration::updateValue($key, $value);


            if (empty($errors) == true) {
                $output .= $this->displayConfirmation('Настройки обновлены!');
            }

            foreach ($errors as $error) {
                $output .= $this->displayError($this->l($error));
            }
        }

        $resultsTest = null;
        $resultsTestMessage = null;
        if (Tools::isSubmit('submit'.$this->name.'check')) {

            $resultsTest = $this->testConnection();
            $resultsTestMessage = $resultsTest[0];
            if ($resultsTest[1]) {
                $output .= $this->displayConfirmation($resultsTestMessage);
            } else {
                $output .= $this->displayError($resultsTestMessage);
            }
        }

        return $output.$this->displayFormCheckConnection($resultsTestMessage).$this->displayForm();
    }

    /**
     * @param string|null $result
     * @return mixed
     */
    private function displayFormCheckConnection ($result)
    {
        $output = '
                <form id="configuration_form_1" class="defaultForm form-horizontal chekonline"
                action="'.AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules').'"
                method="post" enctype="multipart/form-data" novalidate>
                <input type="hidden" name="submitchekonlinecheck" value="1" />
                    <div class="panel" id="fieldset_0_1">
                            <button type="submit" value="1"	id="configuration_form_submit_btn_1" name="submitchekonlinecheck" class="btn btn-primary">
                             Проверить соединение
                            </button>
                    </div>
                </form>';
        return $output;
    }

    /**
     * @return mixed
     */
    private function displayForm ()
    {
        /** @var  $default_lang */

        $op_status = array();

        $sql = 'SELECT * FROM `'._DB_PREFIX_.'order_state_lang`';
        if ($results = Db::getInstance()->ExecuteS($sql)) {
            foreach ($results as $row) {
                $name_id = $row['name'] . ' (id=' . $row['id_order_state'] . ')';
                $add_arr = array('id_option' => $row['id_order_state'], 'name' => $name_id);
                array_push($op_status, $add_arr);
            }
        }

        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => 'Настройки подключения к кассе',
            ),

            'input' => array(



                array(
                    'type' => 'text',
                    'label' => 'Адрес устройства/сервера',
                    'desc' => '
                        Если используется автономная касса:<br>
                        В поле “Адрес устройства/сервера FCE” вписать адрес кассы в сети интернет.
                        Адрес должен иметь вид http://xx.xx.xx.xx:4444. Для работы необходимо
                        пробросить в интернет порт кассы 4444.<br>
                        Если используется облачное решение:<br>
                        В поле “Адрес устройства/сервера FCE” вписать адрес https://kkt.chekonline.ru
                    ',
                    'name' => 'ENDPOINT',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => 'Группа устройств',
                    'desc' => '
                        Группа устройств
                    ',
                    'name' => 'GROUP',
                    'size' => 20
                ),
                array(
                    'type' => 'text',
                    'desc' => '
                        Если с кассой работает один интернет-магазин - поле можно оставить пустым. Если с кассой работает
                        несколько интернет-магазинов - в каждом нужно ввести любую строку, отличную
                        от других магазинов.
                    ',
                    'label' => 'Идентификатор точки продажи',
                    'name' => 'CLIENT_ID',
                    'size' => 20
                ),
                array(
                    'type' => 'text',
                    'label' => 'Пароль кассира',
                    'desc' => '
                    Если вы не меняли Пароль кассира самостоятельно - пароль 1.
                    ',
                    'name' => 'CASHIER_PASSWORD',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => 'Количество документов в смене до автоматического закрытия',
                    'desc' => '
                        Максимально допустимое число — 10000.
                    ',
                    'name' => 'DOCS_IN_TURN',
                    'size' => 20
                ),
                array(
                    'type' => 'text',
                    'label' => 'Пароль к сертификату',
                    'desc' => '
                        Если при генерации сертификата пароль не задавался, то поле не заполняется.
                    ',
                    'name' => 'CERT_PASSWORD',
                    'size' => 20
                ),
                array(
                    'type' => 'textarea',
                    'label' => 'Сертификат',
                    'desc' => '
                        В поле скопировать всё содержимое файла сертификата.
                    ',
                    'name' => 'CERT',
                    'size' => 500
                ),
                array(
                    'type' => 'textarea',
                    'label' => 'Приватный ключ',
                    'desc' => '
                        В поле скопировать всё содержимое файла ключа.
                    ',
                    'name' => 'KEY',
                    'size' => 500
                ),
                array(
                    'type' => 'select',
                    'label' => 'Система налогообложения',
                    'desc' => '
                         Cистема налогообложения.
                    ',
                    'name' => 'TAX_MODE',
                    'size' => 6,
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 0,
                                'name' => 'ОСН'
                            ),
                            array(
                                'id_option' => 1,
                                'name' => 'УСН (доходы)'
                            ),
                            array(
                                'id_option' => 2,
                                'name' => 'УСН (доходы-расходы)'
                            ),
                            array(
                                'id_option' => 3,
                                'name' => 'ЕНВД'
                            ),
                            array(
                                'id_option' => 4,
                                'name' => 'ЕСХН'
                            ),
                            array(
                                'id_option' => 5,
                                'name' => 'ПСН'
                            ),
                        ),
                        'id' => 'id_option',
                        'name' => 'name',
                    ),
                ),

                array(
                    'type' => 'select',
                    'label' => 'НДС на товары',
                    'desc' => '
                        Налог на товары.
                    ',
                    'name' => 'PRODUCT_VAT',
                    'size' => 6,
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 1,
                                'name' => 'НДС 20'
                            ),
                            array(
                                'id_option' => 2,
                                'name' => 'НДС 10'
                            ),
                            array(
                                'id_option' => 3,
                                'name' => 'НДС 0%'
                            ),
                            array(
                                'id_option' => 4,
                                'name' => 'Без НДС'
                            ),
                            array(
                                'id_option' => 5,
                                'name' => 'НДС 20/120'
                            ),
                            array(
                                'id_option' => 6,
                                'name' => 'НДС 10/110'
                            ),
                        ),
                        'id' => 'id_option',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type' => 'select',
                    'label' => 'НДС на доставку',
                    'desc' => '
                        Налог на доставку.
                    ',
                    'name' => 'DELIVERY_VAT',
                    'size' => 6,
                    'required' => true,
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 1,
                                'name' => 'НДС 20'
                            ),
                            array(
                                'id_option' => 2,
                                'name' => 'НДС 10'
                            ),
                            array(
                                'id_option' => 3,
                                'name' => 'НДС 0%'
                            ),
                            array(
                                'id_option' => 4,
                                'name' => 'Без НДС'
                            ),
                            array(
                                'id_option' => 5,
                                'name' => 'НДС 20/120'
                            ),
                            array(
                                'id_option' => 6,
                                'name' => 'НДС 10/110'
                            ),
                        ),
                        'id' => 'id_option',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type' => 'checkbox',
                    'label' => 'Статусы прихода',
                    'desc' => '
                        Настройка статусов заказов, для которых будут печататься чеки прихода.
                    ',
                    'name' => 'STATUS_PAID',
                    'size' => 20,
                    'values' => array(
                        'query' => $op_status,
                        'id'    => 'id_option',
                        'name'  => 'name'
                    ),
                ),
                array(
                    'type' => 'checkbox',
                    'label' => 'Статусы возврата',
                    'desc' => '
                        Настройка статусов заказов, для которых будут печататься чеки возврата.
                    ',
                    'name' => 'STATUS_RETURN',
                    'size' => 20,
                    'values' => array(
                        'query' => $op_status,
                        'id' => 'id_option',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type' => 'checkbox',
                    'label' => 'Логирование',
                    'name' => 'LOG',
                    'size' => 20,
                    'values' => array(
                        'query' => array(array('id_option' => 'LOG', 'name' => 'Включено')),
                        'id' => 'id_option',
                        'name' => 'name',
                    ),
                ),
                array(
                    'type'  => 'text',
                    'label' => $this->l('Cron URL'),
                    'name'  => 'CHEKONLINE_CRON_URL',
                ),
            ),


            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        );

        $helper = new HelperFormCore();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' =>
                array(
                    'desc' => $this->l('Save'),
                    'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                        '&token='.Tools::getAdminTokenLite('AdminModules'),
                ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value['ENDPOINT'] = Configuration::get('ENDPOINT');
        $helper->fields_value['GROUP'] = Configuration::get('GROUP');
        $helper->fields_value['CLIENT_ID'] = Configuration::get('CLIENT_ID');
        $helper->fields_value['CASHIER_PASSWORD'] = Configuration::get('CASHIER_PASSWORD');
        $helper->fields_value['DOCS_IN_TURN'] = Configuration::get('DOCS_IN_TURN');
        $helper->fields_value['CERT_PASSWORD'] = Configuration::get('CERT_PASSWORD');
        $helper->fields_value['CERT'] = Configuration::get('CERT');
        $helper->fields_value['KEY'] = Configuration::get('KEY');
        $helper->fields_value['TAX_MODE'] = Configuration::get('TAX_MODE');
        $helper->fields_value['PRODUCT_VAT'] = Configuration::get('PRODUCT_VAT');
        $helper->fields_value['DELIVERY_VAT'] = Configuration::get('DELIVERY_VAT');
        $helper->fields_value['LOG_LOG'] = Configuration::get('LOG');
        $helper->fields_value['CHEKONLINE_CRON_URL'] = _PS_BASE_URL_._MODULE_DIR_.'chekonline/cron.php?secure_key='. md5(_COOKIE_KEY_ . Configuration::get('PS_SHOP_NAME'));


        $sql = 'SELECT * FROM `'._DB_PREFIX_.'order_state_lang`';
        if ($results = Db::getInstance()->ExecuteS($sql)) {
            foreach ($results as $row) {
                $st_var = 'STATUS_PAID_' . $row['id_order_state'];
                $helper->fields_value[$st_var] = Configuration::get($st_var);


                $st_var = 'STATUS_RETURN_' . $row['id_order_state'];
                $helper->fields_value[$st_var] = Configuration::get($st_var);
            }
        }

        return $helper->generateForm($fields_form);
    }

    /**
     * @param $params
     */
    public function hookActionOrderStatusUpdate ($params)
    {
        $order = new Order(intval($params['id_order']));

        $status = $params['newOrderStatus'];
        if ($this->isStatusForReceipt($status->id)) {

            $type = $this->getType($status->id);
            $receipt = ReceiptModel::getFromTypeAndOrder($type, strval($order->id));

            if (isset($receipt) == false) {
                $receipt = new ReceiptModel();
                $statusId = 0;
                $receipt->order_number = strval($order->id);
                $receipt->type = $type;
                $receipt->status = $statusId;

                $receipt->title[1] = $type ? 'Возврат' : 'Продажа' ;
                $receipt->content[1] = $statusId ? 'Напечатан' : 'В процессе печати';
                $receipt->date_of_printing = strval(date("Y-m-d H:i:s"));
                $receipt->save();
            }
            $this->sendReceipt($order, $receipt);
        }
    }

    /**
     *
     */
    public function resendReceipts()
    {
        $starDate = new DateTime();
        $starDate->sub(new DateInterval('P'.self::NUMBER_OF_DAYS_FOR_RESEND_RECEIPT.'D'));
        $receipts = ReceiptModel::getReceiptsForResend($starDate->format('Y-m-d  H:i:s'));
        foreach ($receipts as $receipt) {
            ChekonlineHelper::writeLog('cron', "resend receipt id: {$receipt->id}; order id: {$receipt->order_number}; type: $receipt->type", $this->logPrefix);
            $order = new Order($receipt->order_number);

            /**
             * Заказ не найден в базе
             */
            if (isset($order->id) == false) {
                ChekonlineHelper::writeLog('cron', "resend receipt order id: {$receipt->order_number}; not found", $this->logPrefix);
                continue;
            }

            if ($this->sendReceipt($order, $receipt) == 1) {
                ChekonlineHelper::writeLog('cron', "success resend receipt order id: {$receipt->order_number};", $this->logPrefix);
            } else {
                ChekonlineHelper::writeLog('cron', "fail resend receipt order id: {$receipt->order_number};", $this->logPrefix);
            }
        }
    }

    /**
     * @param $order
     * @param $receipt
     *
     * @return int
     */
    private function sendReceipt($order, $receipt)
    {

        $statusId = 0;
        try {
            $api = new Api(Configuration::get('ENDPOINT'));
            $api->setClientName('Prestashop');
            $api->setClientVer(_PS_VERSION_);
            $api->setCertificate(
                Configuration::get('CERT'),
                Configuration::get('KEY'),
                Configuration::get('CERT_PASSWORD')
            );
            /**
             * validation of parameters api, can cause an exception
             */
            $api->validate();

            $orderNumber = strval($order->id);

//          todo: chek type
            $type = $receipt->type;
            $documentType = $type ? DocumentType::REFUND_DEBIT : DocumentType::DEBIT;

            $customer = new Customer($order->id_customer);
            $contact = $customer->email;



            $receiptId = strval($receipt->id);

            $complexCommand = new ComplexCommand($receiptId);

            $complexCommand->setPhoneOrEmail($contact)
                           ->setClientId(Configuration::get('CLIENT_ID'))
                           ->setTaxMode(pow(2, intval(Configuration::get('TAX_MODE'))))
                           ->setMaxDocumentsInTurn(intval(Configuration::get('DOCS_IN_TURN')))
                           ->setGroup(Configuration::get('GROUP'))
                           ->setDocumentType($documentType)
                           ->setPassword(intval(Configuration::get('CASHIER_PASSWORD')))
                           ->setFullResponse(false)
                           ->setDevice('auto');

            $orderDetail = new OrderDetail();
            $orderItems = $orderDetail->getList($orderNumber);


            $productVat = intval(Configuration::get('PRODUCT_VAT'));
            foreach ($orderItems as $orderItem) {
                $qty = $orderItem['product_quantity'] * 1000;
                $subtotal = round($orderItem['total_price_tax_incl'] * 100);
                if ($subtotal <= 0) {
                    continue;
                }
                $description = $orderItem['product_name'];

                $receiptLine = new Line();
                $receiptLine->setDescription($description)
                            ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                            ->setSubTotal($subtotal)
                            ->setQty($qty)
                            ->setTaxId($productVat);
                /**
                 * validation of parameters line before adding to the receipt, can cause an exception
                 */
                $receiptLine->validate();
                $complexCommand->addLine($receiptLine);
            }

            $deliveryVat = round(Configuration::get('DELIVERY_VAT'));
            if (isset($order->total_shipping_tax_incl) && floatval($order->total_shipping_tax_incl) > 0) {
                $qty = 1000;
                $subtotal = round($order->total_shipping_tax_incl * 100);
                $description = 'Доставка';
                $isShipping = true;
                $receiptLine = new Line($isShipping);

                $receiptLine->setDescription($description)
                            ->setPayAttribute(PayAttribute::FULL_PAYMENT)
                            ->setSubTotal($subtotal)
                            ->setQty($qty)
                            ->setTaxId($deliveryVat);
                /**
                 * validation of parameters line before adding to the receipt, can cause an exception
                 */
                $receiptLine->validate();
                $complexCommand->addLine($receiptLine);
            }


            $orderTotal =  round($order->total_paid_tax_incl * 100);
            $complexCommand->normalizeBySubTotal($orderTotal);
            $complexCommand->setNonCash(array(0, $orderTotal, 0));

            $complexCommand->validate();

            $api->validate();

            $response = $api->executeCommand($complexCommand);
            $request = $response->getRequest();

            if ($this->isReceiptSuccess($response)) {
                $statusId = 1;
            }

            $receipt->status = $statusId;
            $receipt->content[1] = $statusId ? 'Напечатан' : 'В процессе печати';
            $receipt->save();

            $log = Configuration::get('LOG');


            if ( isset($log) == true && empty($log) == false) {
                ChekonlineHelper::writeLog('request', $request, $this->logPrefix);
                ChekonlineHelper::writeLog('response', $response, $this->logPrefix);
            }

            if ($this->isReceiptSuccess($response) == false) {
                ChekonlineHelper::writeLog('error', $response, $this->logPrefix);
            }
        } catch (ChekonlineLineException $exception) {
            $logMessage = "Chekonline line receipt exception:\n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nLine:\n" . var_export($exception->getReceiptLine()->getParam(), true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineHelper::writeLog('fail', $logMessage, $this->logPrefix);
        } catch (ChekonlineCommandException $exception) {
            $logMessage = "Chekonline command exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nCommand:" . var_export($exception->getCommand()->getParams(), true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineHelper::writeLog('fail', $logMessage, $this->logPrefix);
        } catch (ChekonlineSDKException $exception) {
            $logMessage = "Chekonline exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineHelper::writeLog('fail', $logMessage, $this->logPrefix);
        }
        return $statusId;
    }

    /**
     * @param int $statusId
     * @return bool
     */
    private function isStatusForReceipt ($statusId)
    {
        $status = Configuration::get('STATUS_PAID_'.$statusId);
            if (isset($status) === false || empty($status) === true){
            $status = Configuration::get('STATUS_RETURN_' . $statusId);
            if (isset($status) === false || empty($status) === true) {
                return false;
            }
        }
        return true;
    }

    /**
     * @param int $statusId
     * @return int
     */
    private function getType($statusId)
    {
        $status = Configuration::get('STATUS_PAID_'.$statusId);

        if ($status == 'on'){
            return 0;
        } else {
            return 1;
        }
    }

    /**
     * @param \Chekonline\Cashbox\ChekonlineResponse $apiResponse
     * @return bool
     */
    private function isReceiptSuccess($apiResponse)
    {
        if ($apiResponse->getHttpStatusCode() == 200 &&
            $apiResponse->getApiErrorCode() == 0 &&
            $apiResponse->getHttpError() == '') {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @return mixed
     */
    private function testConnection()
    {
	    $result = false;
        try {
            $api = new Api(Configuration::get('ENDPOINT'));
            $api->setClientName('Prestashop');
            $api->setClientVer(_PS_VERSION_);
            $api->setCertificate(
                Configuration::get('CERT'),
                Configuration::get('KEY'),
                Configuration::get('CERT_PASSWORD')
            );

            $requestId = strval(time());
            $batchCommand = new BatchCommand($requestId);

            $batchCommand->setQueueLen(100)
                ->setDevice('auto')
                ->setClientId(Configuration::get('CLIENT_ID'));


            $noOperationCommand = new NoOperationCommand($requestId);

            $noOperationCommand->setPassword(intval(Configuration::get('CASHIER_PASSWORD')))
                               ->setClientId(Configuration::get('CLIENT_ID'));

            $batchCommand->addCommandForExecute($noOperationCommand);

            $noOperationCommand->validate();
            $batchCommand->validate();
            $api->validate();

            $apiResponse =  $api->executeCommand($batchCommand);


            if ($this->isReceiptSuccess($apiResponse) == true) {
                $resultMessage ='Соединение установлено';
                $result = true;
            } else {
                $resultMessage = "Код ответа: 
                {$apiResponse->getApiErrorCode()}
                \nОтвет: {$apiResponse->getApiErrorMessage()}
                 {$apiResponse}";
            }

            $log = Configuration::get('LOG');


            if ( isset($log) == true && empty($log) == false) {
                ChekonlineHelper::writeLog('request', $apiResponse->getRequest(), $this->logPrefix);
                ChekonlineHelper::writeLog('response', $apiResponse, $this->logPrefix);
            }

            if ($this->isReceiptSuccess($apiResponse) == false) {
                ChekonlineHelper::writeLog('error', $apiResponse, $this->logPrefix);
            }

        } catch (ChekonlineLineException $exception) {
            $logMessage = "Chekonline line receipt exception:\n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nLine:\n".var_export($exception->getReceiptLine()->getParam(),true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineHelper::writeLog('fail', $logMessage, $this->logPrefix);
            $resultMessage = $exception->getMessage();
        } catch (ChekonlineCommandException $exception) {
            $logMessage = "Chekonline command exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nCommand:".var_export($exception->getCommand()->getParams(),true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineHelper::writeLog('fail', $logMessage, $this->logPrefix);
            $resultMessage = $exception->getMessage();
        } catch (ChekonlineSDKException $exception) {
            $logMessage = "Chekonline exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineHelper::writeLog('fail', $logMessage, $this->logPrefix);
            $resultMessage = $exception->getMessage();
        }
        return array($resultMessage, $result);
    }
}